@@ -1,6 +1,6 @@
 /* gui.h
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -157,7 +157,7 @@ void gui_sectionlist_buttons_refresh(struct gui *);
 void gui_messagelist_refresh(struct gui *, gboolean);
 void gui_trackvolumes_refresh(struct gui *);
 void gui_preferences_refresh(struct gui *);
-void gui_tracker_refresh(struct gui *);
+void gui_tracker_refresh(struct gui *, gboolean);
 void gui_song_refresh(struct gui *);
 void gui_receive_refresh(struct gui *, int, int);
   
