@@ -1,6 +1,6 @@
 /* block.c
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -303,26 +303,31 @@ void block_paste(struct block *block, struct block *from, int track, int line)
   nullcheck_void(block, block_paste);
   nullcheck_void(from, block_paste);
 
-  int t, l, ep, nl, nt, fnl, fnt, cnl, cnt;
+  int t, l, ep, nl, nt, nc, fnl, fnt, fnc, cnl, cnt, cnc;
 
   nl = block->length;
   nt = block->tracks;
+  nc = block->commandpages;
   fnl = from->length;
   fnt = from->tracks;
+  fnc = from->commandpages;
   cnl = fnl;
   cnt = fnt;
+  cnc = fnc;
 
   if (line + fnl > nl)
     cnl = nl - line;
   if (track + fnt > nt)
     cnt = nt - track;
+  if (cnc > nc)
+    cnc = nc;
 
   /* Copy the from block to the destination block; make sure it fits */
   for (l = 0; l < cnl; l++) {
     for (t = 0; t < cnt; t++) {
       block->notes[((line + l) * nt + (track + t)) * 2] = from->notes[(l * fnt + t) * 2];
       block->notes[((line + l) * nt + (track + t)) * 2 + 1] = from->notes[(l * fnt + t) * 2 + 1];
-      for (ep = 0; ep < block->commandpages; ep++) {
+      for (ep = 0; ep < cnc; ep++) {
 	block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2];
 	block->commands[ep * 2 * nt * nl + ((line + l) * nt + (track + t)) * 2 + 1] = from->commands[ep * 2 * fnt * fnl + (l * fnt + t) * 2 + 1];
       }
