@@ -1,6 +1,6 @@
 /* callbacks.c
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -560,7 +560,7 @@ void on_menuitem_about_activate(GtkMenuItem * menuitem, gpointer user_data)
     return;
   }
 
-  gui->window_about = gnome_about_new("Tutka", VERSION, "Copyright \xc2\xa9 2002-2005 Vesa Halttunen", "http://www.nongnu.org/tutka/", (const gchar **)authors, (const gchar **)authors, NULL, NULL);
+  gui->window_about = gnome_about_new("Tutka", VERSION, "Copyright \xc2\xa9 2002-2006 Vesa Halttunen", "http://www.nongnu.org/tutka/", (const gchar **)authors, (const gchar **)authors, NULL, NULL);
 
   gtk_window_set_destroy_with_parent(GTK_WINDOW(gui->window_about), TRUE);
   g_signal_connect(G_OBJECT(gui->window_about), "destroy", G_CALLBACK(gtk_widget_destroyed), &gui->window_about);
@@ -1029,7 +1029,7 @@ void on_button_blocklist_insertnew_clicked(GtkButton * button, gpointer user_dat
   struct gui *gui = (struct gui *)user_data;
 
   editor_song_block_insert(gui->editor, gui->editor->block, gui->editor->block);
-  gui_tracker_refresh(gui);
+  gui_tracker_refresh(gui, FALSE);
   gui_blocklist_refresh(gui, TRUE);
   gui_playseq_refresh(gui, TRUE);
   gui_info_refresh(gui);
@@ -1057,7 +1057,7 @@ void on_button_blocklist_delete_clicked(GtkButton * button, gpointer user_data)
 
   /* Go back to the line the editor was on */
   gui->editor->line = line;
-  gui_tracker_refresh(gui);
+  gui_tracker_refresh(gui, FALSE);
 }
 
 void on_entry_instrumentproperties_name_changed(GtkEditable * widget, gpointer user_data)
@@ -1886,13 +1886,13 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
       if (editor_song_maxtracks_check(gui->editor))
 	gui_trackvolumes_refresh(gui);
 
-      gui_tracker_refresh(gui);
+      gui_tracker_refresh(gui, FALSE);
       break;
     }
   case GUI_BLOCKLIST_COLUMN_LENGTH:
     {
       editor_song_block_current_length_set(gui->editor, value);
-      gui_tracker_refresh(gui);
+      gui_tracker_refresh(gui, TRUE);
 
       break;
     }
@@ -1901,7 +1901,7 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
       editor_song_block_current_commandpages_set(gui->editor, value);
       if (gui->editor->cmdpage >= editor_song_block_current_commandpages_get(gui->editor)) {
 	gui->editor->cmdpage = editor_song_block_current_commandpages_get(gui->editor) - 1;
-	gui_tracker_refresh(gui);
+	gui_tracker_refresh(gui, FALSE);
       }
       gui_info_refresh(gui);
       break;
