@@ -1,6 +1,6 @@
 /* editor.c
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -397,7 +397,7 @@ void editor_set_line(struct editor *editor, int line)
     /* If called from GUI set player position, always refresh GUI */
     if (g_thread_self() == editor->thread) {
       player_set_line(editor->player, line);
-      gui_tracker_refresh(editor->gui);
+      gui_tracker_refresh(editor->gui, FALSE);
     } else
       gui_set_refresh(editor->gui, GUI_REFRESH_TRACKER);
   }
@@ -1023,6 +1023,9 @@ void editor_song_track_mute_set(struct editor *editor, unsigned int track, unsig
   player_lock(editor->player);
   player_stop_muted(editor->player);
   player_unlock(editor->player);
+  
+  /* Refresh the tracker */
+  gui_tracker_refresh(editor->gui, FALSE);
 }
 
 /* Returns the muting of a track */
@@ -1050,6 +1053,9 @@ void editor_song_track_solo_set(struct editor *editor, unsigned int track, unsig
   player_lock(editor->player);
   player_stop_muted(editor->player);
   player_unlock(editor->player);
+  
+  /* Refresh the tracker */
+  gui_tracker_refresh(editor->gui, FALSE);
 }
 
 /* Returns the soloing of a track */
@@ -1073,7 +1079,7 @@ void editor_song_track_name_set(struct editor *editor, unsigned int track, char
   g_mutex_unlock(editor->songmutex);
 
   /* Refresh the tracker */
-  gui_tracker_refresh(editor->gui);
+  gui_tracker_refresh(editor->gui, FALSE);
 }
 
 /* Returns the name of a track */
@@ -1161,6 +1167,7 @@ void editor_song_block_current_copyblock_paste(struct editor *editor)
 unsigned int editor_song_nummessages_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_song_nummessages_get);
+  nullcheck_int(editor->song, editor_song_nummessages_get);
 
   return editor->song->nummessages;
 }
