@@ -1,6 +1,6 @@
 /* gui.c
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -490,14 +490,14 @@ void gui_refresh(struct gui *gui)
 
   g_mutex_lock(gui->refresh_mutex);
   if (gui->refresh_mask & GUI_REFRESH_TRACKER) {
-    gui_tracker_refresh(gui);
+    gui_tracker_refresh(gui, FALSE);
     gui->refresh_mask ^= GUI_REFRESH_TRACKER;
   }
   g_mutex_unlock(gui->refresh_mutex);
 }
 
 /* Refresh the tracker */
-void gui_tracker_refresh(struct gui *gui)
+void gui_tracker_refresh(struct gui *gui, gboolean redraw)
 {
   Tracker *tracker = gui_get_tracker(gui);
   unsigned int length = editor_song_block_current_length_get(gui->editor);
@@ -515,6 +515,11 @@ void gui_tracker_refresh(struct gui *gui)
   tracker_set_pattern(tracker, editor_song_block_current_get(gui->editor));
   tracker_set_patpos(tracker, gui->editor->line);
   tracker_set_cmdpage(tracker, gui->editor->cmdpage);
+  tracker_idle_draw(tracker);
+
+  if (redraw) {
+    tracker_redraw(tracker);
+  }
 }
 
 /* Refresh the song properties window widgets */
@@ -918,14 +923,16 @@ void gui_messagelist_refresh(struct gui *gui, gboolean recreate)
   char oldmessage[10];
   GtkTreeView *treeview;
 
-  if (editor_song_nummessages_get(gui->editor) == 0)
+  if (gui->editor->song == NULL || editor_song_nummessages_get(gui->editor) == 0)
     gui->messagelist_message = -1;
 
   snprintf(oldmessage, 10, "%d", gui->messagelist_message);
   treeview = GTK_TREE_VIEW(glade_xml_get_widget(gui->xml, "treeview_messagelist"));
 
   if (recreate) {
-    int n = editor_song_nummessages_get(gui->editor);
+    int n = 0;
+    if (gui->editor->song != NULL)
+      n = editor_song_nummessages_get(gui->editor);
 
     /* Recreate the Message list */
     GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
