@@ -1,6 +1,6 @@
 /* player.c
  *
- * Copyright 2002-2005 Vesa Halttunen
+ * Copyright 2002-2006 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -136,7 +136,7 @@ void player_stop_muted(struct player *player)
 
   struct song *song = player->song;
   int i, j;
-    
+
   for (i = 0; i < song->maxtracks; i++)
     if (song->tracks[i]->mute || (player->solo && !song->tracks[i]->solo)) {
       struct player_track_status *trackstatus = player->trackstatus[i];
@@ -284,7 +284,7 @@ static void player_thread(struct player *player)
 	/* Wait for a sync signal to come in */
 	g_cond_wait(player->external_sync, player->mutex);
       if (player->external_sync_ticks > 0)
-        player->external_sync_ticks--;
+	player->external_sync_ticks--;
     } else
       prevsched = player->sched;
 
@@ -305,41 +305,44 @@ static void player_thread(struct player *player)
     /* Stop notes if there are new notes about to be played */
     for (i = 0; i < b->tracks; i++) {
       struct player_track_status *trackstatus = player->trackstatus[i];
-      int delay = 0;
-      unsigned char note = b->notes[(b->tracks * player->line + i) * 2];
 
-      /* Notes are played if the track is not muted and no tracks are soloed or the current track is soloed */
-      if (note != 0 && !song->tracks[i]->mute && (!player->solo || (player->solo && song->tracks[i]->solo))) {
-	for (j = 0; j < b->commandpages; j++) {
-	  unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2];
-	  unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2 + 1];
+      /* The track is taken into account if the track is not muted and no tracks are soloed or the current track is soloed */
+      if (!song->tracks[i]->mute && (!player->solo || (player->solo && song->tracks[i]->solo))) {
+        int delay = 0;
+        unsigned char note = b->notes[(b->tracks * player->line + i) * 2];
+        
+	if (note != 0) {
+	  for (j = 0; j < b->commandpages; j++) {
+	    unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2];
+	    unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2 + 1];
 
-	  if (command != 0 || value != 0) {
-	    /* Check for previous command if any */
-	    if (command == COMMAND_PREVIOUS_COMMAND_VALUE) {
-	      if (value != 0)
-		command = trackstatus->previouscommand;
-	    } else
-	      trackstatus->previouscommand = command;
+	    if (command != 0 || value != 0) {
+	      /* Check for previous command if any */
+	      if (command == COMMAND_PREVIOUS_COMMAND_VALUE) {
+		if (value != 0)
+		  command = trackstatus->previouscommand;
+	      } else
+		trackstatus->previouscommand = command;
 
-	    switch (command) {
-	    case COMMAND_RETRIGGER:
-	      delay = -value;
-	      break;
-	    case COMMAND_DELAY:
-	      delay = value;
-	      break;
+	      switch (command) {
+	      case COMMAND_RETRIGGER:
+		delay = -value;
+		break;
+	      case COMMAND_DELAY:
+		delay = value;
+		break;
+	      }
 	    }
 	  }
-	}
 
-	/* Stop currently playing note */
-	if ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0)) {
-	  for (j = 0; j < trackstatus->numinterfaces; j++) {
-	    struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
-	    if (interfacestatus->note != -1) {
-	      midi_note_off(player->midi->outputs[j], interfacestatus->midichannel, interfacestatus->note, 127);
-	      interfacestatus->note = -1;
+	  /* Stop currently playing note */
+	  if ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0)) {
+	    for (j = 0; j < trackstatus->numinterfaces; j++) {
+	      struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
+	      if (interfacestatus->note != -1) {
+		midi_note_off(player->midi->outputs[j], interfacestatus->midichannel, interfacestatus->note, 127);
+		interfacestatus->note = -1;
+	      }
 	    }
 	  }
 	}
@@ -349,250 +352,255 @@ static void player_thread(struct player *player)
     /* Play notes scheduled to be played */
     for (i = 0; i < b->tracks; i++) {
       struct player_track_status *trackstatus = player->trackstatus[i];
-      unsigned int volume = 127;
-      int delay = 0, hold = -1;
-      unsigned char note = b->notes[(b->tracks * player->line + i) * 2];
-      unsigned char instrument = b->notes[(b->tracks * player->line + i) * 2 + 1];
 
-      /* Handle commands on all command pages */
-      for (j = 0; j < b->commandpages; j++) {
-	unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2];
-	unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2 + 1];
+      /* The track is taken into account if the track is not muted and no tracks are soloed or the current track is soloed */
+      if (!song->tracks[i]->mute && (!player->solo || (player->solo && song->tracks[i]->solo))) {
+	unsigned int volume = 127;
+	int delay = 0, hold = -1;
+	unsigned char note = b->notes[(b->tracks * player->line + i) * 2];
+	unsigned char instrument = b->notes[(b->tracks * player->line + i) * 2 + 1];
 
-	if (command != 0 || value != 0) {
-	  unsigned int *outputs;
-	  unsigned int numoutputs;
+	/* Handle commands on all command pages */
+	for (j = 0; j < b->commandpages; j++) {
+	  unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2];
+	  unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2 + 1];
 
-	  /* Check which MIDI interface/channel pairs the command will affect */
-	  if (instrument != 0) {
-	    /* Instrument number defines MIDI interfaces/channels */
-	    numoutputs = song->instruments[instrument - 1]->numoutputs;
-	    outputs = (unsigned int *)calloc(numoutputs * 2, sizeof(unsigned int));
-	    for (k = 0; k < numoutputs; k++) {
-	      outputs[k * 2 + 0] = song->instruments[instrument - 1]->outputs[k]->midiinterface;
-	      outputs[k * 2 + 1] = song->instruments[instrument - 1]->outputs[k]->midichannel;
+	  if (command != 0 || value != 0) {
+	    unsigned int *outputs;
+	    unsigned int numoutputs;
+
+	    /* Check which MIDI interface/channel pairs the command will affect */
+	    if (instrument != 0) {
+	      /* Instrument number defines MIDI interfaces/channels */
+	      numoutputs = song->instruments[instrument - 1]->numoutputs;
+	      outputs = (unsigned int *)calloc(numoutputs * 2, sizeof(unsigned int));
+	      for (k = 0; k < numoutputs; k++) {
+		outputs[k * 2 + 0] = song->instruments[instrument - 1]->outputs[k]->midiinterface;
+		outputs[k * 2 + 1] = song->instruments[instrument - 1]->outputs[k]->midichannel;
+	      }
+	    } else {
+	      /* Note playing defines MIDI interfaces/channels */
+	      numoutputs = player->trackstatus[i]->numinterfaces;
+	      outputs = (unsigned int *)calloc(numoutputs * 2, sizeof(unsigned int));
+	      for (k = 0; k < numoutputs; k++) {
+		outputs[k * 2 + 0] = k;
+		outputs[k * 2 + 1] = player->trackstatus[i]->interfaces[k]->midichannel;
+	      }
 	    }
-	  } else {
-	    /* Note playing defines MIDI interfaces/channels */
-	    numoutputs = player->trackstatus[i]->numinterfaces;
-	    outputs = (unsigned int *)calloc(numoutputs * 2, sizeof(unsigned int));
+
+	    /* Handle all relevant MIDI interfaces/channels */
 	    for (k = 0; k < numoutputs; k++) {
-	      outputs[k * 2 + 0] = k;
-	      outputs[k * 2 + 1] = player->trackstatus[i]->interfaces[k]->midichannel;
-	    }
-	  }
+	      int midiinterface = outputs[k * 2 + 0];
+	      int midichannel = outputs[k * 2 + 1];
+	      struct midi_interface *midi = player->midi->outputs[midiinterface];
+	      struct player_interface_status *interfacestatus = trackstatus->interfaces[midiinterface];
 
-	  /* Handle all relevant MIDI interfaces/channels */
-	  for (k = 0; k < numoutputs; k++) {
-	    int midiinterface = outputs[k * 2 + 0];
-	    int midichannel = outputs[k * 2 + 1];
-	    struct midi_interface *midi = player->midi->outputs[midiinterface];
-	    struct player_interface_status *interfacestatus = trackstatus->interfaces[midiinterface];
-
-	    /* Check for previous command if any */
-	    if (command == COMMAND_PREVIOUS_COMMAND_VALUE) {
-	      if (value != 0)
-		command = trackstatus->previouscommand;
-	    } else
-	      trackstatus->previouscommand = command;
-
-	    switch (command) {
-	    case COMMAND_PITCH_WHEEL:
-	      /* Program change can be sent if the MIDI channel is known */
-	      if (midichannel != -1) {
-		if (value < 0x80) {
-		  if (player->tick == 0) {
-		    midi_pitch_wheel(midi, midichannel, value);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] = value;
-		  }
-		} else {
-		  if (player->tick < song->ticksperline - 1) {
-		    int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL]) / ((int)song->ticksperline - 1);
-
-		    midi_pitch_wheel(midi, midichannel, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] + player->tick * delta);
-		  } else {
-		    midi_pitch_wheel(midi, midichannel, value - 0x80);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] = value - 0x80;
-		  }
-		}
-	      }
-	      break;
-	    case COMMAND_PROGRAM_CHANGE:
-	      /* Pitch wheel can be set if the MIDI channel is known */
-	      if (midichannel != -1 && player->tick == 0)
-		midi_program_change(midi, midichannel, value & 0x7f);
-	      break;
-	    case COMMAND_END_BLOCK:
-	      /* Only on last tick */
-	      if (player->tick == song->ticksperline - 1) {
-		postcommand = COMMAND_END_BLOCK;
-		postvalue = value;
-	      }
-	      break;
-	    case COMMAND_PLAYSEQ_POSITION:
-	      /* Only on last tick */
-	      if (player->tick == song->ticksperline - 1) {
-		postcommand = COMMAND_PLAYSEQ_POSITION;
-		postvalue = value;
-	      }
-	      break;
-	    case COMMAND_SEND_MESSAGE:
-	      /* Only on first tick */
-	      if (player->tick == 0 && value < song->nummessages)
-		midi_write_raw(midi, song->messages[value]->data, song->messages[value]->length);
-	      break;
-	    case COMMAND_HOLD:
-	      hold = value;
-	      break;
-	    case COMMAND_RETRIGGER:
-	      delay = -value;
-	      break;
-	    case COMMAND_DELAY:
-	      delay = value;
-	      break;
-	    case COMMAND_VELOCITY:
-	      if (note != 0) {
-		volume = value;
-		if (midichannel != -1)
-		  player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value;
-	      } else {
-		/* Note playing defines MIDI channel */
-		midichannel = interfacestatus->midichannel;
+	      /* Check for previous command if any */
+	      if (command == COMMAND_PREVIOUS_COMMAND_VALUE) {
+		if (value != 0)
+		  command = trackstatus->previouscommand;
+	      } else
+		trackstatus->previouscommand = command;
 
+	      switch (command) {
+	      case COMMAND_PITCH_WHEEL:
+		/* Program change can be sent if the MIDI channel is known */
 		if (midichannel != -1) {
 		  if (value < 0x80) {
 		    if (player->tick == 0) {
-		      if (value > 0) {
-			midi_aftertouch(midi, midichannel, interfacestatus->note, value);
-			player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value;
-		      } else {
-			midi_note_off(midi, midichannel, interfacestatus->note, 127);
-			interfacestatus->note = -1;
-		      }
+		      midi_pitch_wheel(midi, midichannel, value);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] = value;
 		    }
 		  } else {
 		    if (player->tick < song->ticksperline - 1) {
-		      int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH]) / ((int)song->ticksperline - 1);
+		      int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL]) / ((int)song->ticksperline - 1);
 
-		      midi_aftertouch(midi, midichannel, interfacestatus->note, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] + player->tick * delta);
+		      midi_pitch_wheel(midi, midichannel, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] + player->tick * delta);
 		    } else {
-		      midi_aftertouch(midi, midichannel, interfacestatus->note, value - 0x80);
-		      player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value - 0x80;
+		      midi_pitch_wheel(midi, midichannel, value - 0x80);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_PITCH_WHEEL] = value - 0x80;
+		    }
+		  }
+		}
+		break;
+	      case COMMAND_PROGRAM_CHANGE:
+		/* Pitch wheel can be set if the MIDI channel is known */
+		if (midichannel != -1 && player->tick == 0)
+		  midi_program_change(midi, midichannel, value & 0x7f);
+		break;
+	      case COMMAND_END_BLOCK:
+		/* Only on last tick */
+		if (player->tick == song->ticksperline - 1) {
+		  postcommand = COMMAND_END_BLOCK;
+		  postvalue = value;
+		}
+		break;
+	      case COMMAND_PLAYSEQ_POSITION:
+		/* Only on last tick */
+		if (player->tick == song->ticksperline - 1) {
+		  postcommand = COMMAND_PLAYSEQ_POSITION;
+		  postvalue = value;
+		}
+		break;
+	      case COMMAND_SEND_MESSAGE:
+		/* Only on first tick */
+		if (player->tick == 0 && value < song->nummessages)
+		  midi_write_raw(midi, song->messages[value]->data, song->messages[value]->length);
+		break;
+	      case COMMAND_HOLD:
+		hold = value;
+		break;
+	      case COMMAND_RETRIGGER:
+		delay = -value;
+		break;
+	      case COMMAND_DELAY:
+		delay = value;
+		break;
+	      case COMMAND_VELOCITY:
+		if (note != 0) {
+		  volume = value;
+		  if (midichannel != -1)
+		    player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value;
+		} else {
+		  /* Note playing defines MIDI channel */
+		  midichannel = interfacestatus->midichannel;
+
+		  if (midichannel != -1) {
+		    if (value < 0x80) {
+		      if (player->tick == 0) {
+			if (value > 0) {
+			  midi_aftertouch(midi, midichannel, interfacestatus->note, value);
+			  player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value;
+			} else {
+			  midi_note_off(midi, midichannel, interfacestatus->note, 127);
+			  interfacestatus->note = -1;
+			}
+		      }
+		    } else {
+		      if (player->tick < song->ticksperline - 1) {
+			int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH]) / ((int)song->ticksperline - 1);
+
+			midi_aftertouch(midi, midichannel, interfacestatus->note, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] + player->tick * delta);
+		      } else {
+			midi_aftertouch(midi, midichannel, interfacestatus->note, value - 0x80);
+			player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_AFTERTOUCH] = value - 0x80;
+		      }
+		    }
+		  }
+		}
+		break;
+	      case COMMAND_CHANNEL_PRESSURE:
+		/* Channel pressure can be set if the MIDI channel is known */
+		if (midichannel != -1) {
+		  if (value < 0x80) {
+		    if (player->tick == 0) {
+		      midi_channel_pressure(midi, midichannel, value);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] = value;
+		    }
+		  } else {
+		    if (player->tick < song->ticksperline - 1) {
+		      int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE]) / ((int)song->ticksperline - 1);
+
+		      midi_channel_pressure(midi, midichannel, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] + player->tick * delta);
+		    } else {
+		      midi_channel_pressure(midi, midichannel, value - 0x80);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] = value - 0x80;
+		    }
+		  }
+		}
+		break;
+	      case COMMAND_TPL:
+		if (value == 0) {
+		  /* Only on last tick */
+		  if (player->tick == song->ticksperline - 1)
+		    postcommand = COMMAND_TPL;
+		} else {
+		  song_set_tpl(song, value);
+		  changetempo = 1;
+		}
+		break;
+	      case COMMAND_TEMPO:
+		if (value == 0) {
+		  /* Only on last tick */
+		  if (player->tick == song->ticksperline - 1)
+		    postcommand = COMMAND_TEMPO;
+		} else {
+		  song_set_tempo(song, value);
+		  midi_tempo(midi, value);
+		  changetempo = 1;
+		}
+		break;
+	      }
+
+	      /* Handle MIDI controllers */
+	      if (command >= COMMAND_MIDI_CONTROLLERS) {
+		/* MIDI controllers can be set if the MIDI channel is known */
+		if (midichannel != -1) {
+		  if (value < 0x80) {
+		    if (player->tick == 0) {
+		      midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, value);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] = value;
+		    }
+		  } else {
+		    if (player->tick < song->ticksperline - 1) {
+		      int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS]) / ((int)song->ticksperline - 1);
+
+		      midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] + player->tick * delta);
+		    } else {
+		      midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, value - 0x80);
+		      player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] = value - 0x80;
 		    }
 		  }
 		}
 	      }
-	      break;
-	    case COMMAND_CHANNEL_PRESSURE:
-	      /* Channel pressure can be set if the MIDI channel is known */
-	      if (midichannel != -1) {
-		if (value < 0x80) {
-		  if (player->tick == 0) {
-		    midi_channel_pressure(midi, midichannel, value);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] = value;
-		  }
-		} else {
-		  if (player->tick < song->ticksperline - 1) {
-		    int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE]) / ((int)song->ticksperline - 1);
-
-		    midi_channel_pressure(midi, midichannel, player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] + player->tick * delta);
-		  } else {
-		    midi_channel_pressure(midi, midichannel, value - 0x80);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + VALUES_CHANNEL_PRESSURE] = value - 0x80;
-		  }
-		}
-	      }
-	      break;
-	    case COMMAND_TPL:
-	      if (value == 0) {
-		/* Only on last tick */
-		if (player->tick == song->ticksperline - 1)
-		  postcommand = COMMAND_TPL;
-	      } else {
-		song_set_tpl(song, value);
-		changetempo = 1;
-	      }
-	      break;
-	    case COMMAND_TEMPO:
-	      if (value == 0) {
-		/* Only on last tick */
-		if (player->tick == song->ticksperline - 1)
-		  postcommand = COMMAND_TEMPO;
-	      } else {
-		song_set_tempo(song, value);
-		midi_tempo(midi, value);
-		changetempo = 1;
-	      }
-	      break;
 	    }
-
-	    /* Handle MIDI controllers */
-	    if (command >= COMMAND_MIDI_CONTROLLERS) {
-	      /* MIDI controllers can be set if the MIDI channel is known */
-	      if (midichannel != -1) {
-		if (value < 0x80) {
-		  if (player->tick == 0) {
-		    midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, value);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] = value;
-		  }
-		} else {
-		  if (player->tick < song->ticksperline - 1) {
-		    int delta = (value - 0x80 - player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS]) / ((int)song->ticksperline - 1);
-
-		    midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] + player->tick * delta);
-		  } else {
-		    midi_controller(midi, midichannel, command - COMMAND_MIDI_CONTROLLERS, value - 0x80);
-		    player->midicontrollervalues[midiinterface][midichannel * VALUES + command - COMMAND_MIDI_CONTROLLERS] = value - 0x80;
-		  }
-		}
-	      }
-	    }
-	  }
-	  free(outputs);
-	}
-      }
-
-      /* Is there a note to play? */
-      if (note != 0 && ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0))) {
-	struct instrument *instr = NULL;
-
-	note--;
-
-	/* Use previous instrument if none defined */
-	if (instrument == 0)
-	  instrument = trackstatus->instrument + 1;
-
-	/* Play note if instrument is defined */
-	if (instrument != 0)
-	  player_play_note(player, instrument - 1, note, volume, i);
-
-	if (instrument <= song->numinstruments)
-	  instr = song->instruments[instrument - 1];
-
-	if (instr != NULL) {
-	  for (j = 0; j < instr->numoutputs; j++) {
-	    struct instrument_output *output = instr->outputs[j];
-	    struct player_interface_status *interfacestatus = trackstatus->interfaces[output->midiinterface];
-
-	    /* If no hold value was defined use the instrument's hold value */
-	    if (hold == -1)
-	      hold = output->hold;
-
-	    if (hold == 0)
-	      interfacestatus->hold = -1;
-	    else
-	      interfacestatus->hold = hold;
+	    free(outputs);
 	  }
 	}
-      }
 
-      /* First tick, no note but instrument defined? */
-      for (j = 0; j < trackstatus->numinterfaces; j++) {
-	struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
-	if (player->tick == 0 && note == 0 && instrument > 0 && interfacestatus->hold >= 0)
-	  for (k = 0; k < song->instruments[instrument - 1]->numoutputs; k++)
-	    if (song->instruments[instrument - 1]->outputs[k]->midiinterface == j)
-	      interfacestatus->hold += song->instruments[instrument - 1]->outputs[k]->hold;
+	/* Is there a note to play? */
+	if (note != 0 && ((delay >= 0 && player->tick == delay) || (delay < 0 && player->tick % (-delay) == 0))) {
+	  struct instrument *instr = NULL;
+
+	  note--;
+
+	  /* Use previous instrument if none defined */
+	  if (instrument == 0)
+	    instrument = trackstatus->instrument + 1;
+
+	  /* Play note if instrument is defined */
+	  if (instrument != 0) {
+	    player_play_note(player, instrument - 1, note, volume, i);
+
+            if (instrument <= song->numinstruments)
+              instr = song->instruments[instrument - 1];
+          }
+
+	  if (instr != NULL) {
+	    for (j = 0; j < instr->numoutputs; j++) {
+	      struct instrument_output *output = instr->outputs[j];
+	      struct player_interface_status *interfacestatus = trackstatus->interfaces[output->midiinterface];
+
+	      /* If no hold value was defined use the instrument's hold value */
+	      if (hold == -1)
+		hold = output->hold;
+
+	      if (hold == 0)
+		interfacestatus->hold = -1;
+	      else
+		interfacestatus->hold = hold;
+	    }
+	  }
+	}
+
+	/* First tick, no note but instrument defined? */
+	for (j = 0; j < trackstatus->numinterfaces; j++) {
+	  struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
+	  if (player->tick == 0 && note == 0 && instrument > 0 && interfacestatus->hold >= 0)
+	    for (k = 0; k < song->instruments[instrument - 1]->numoutputs; k++)
+	      if (song->instruments[instrument - 1]->outputs[k]->midiinterface == j)
+		interfacestatus->hold += song->instruments[instrument - 1]->outputs[k]->hold;
+	}
       }
     }
 
@@ -928,11 +936,11 @@ void player_stop(struct player *player)
     g_mutex_lock(player->mutex);
     player->killthread = 1;
     g_mutex_unlock(player->mutex);
-    
+
     /* If external sync is used send sync to get out of the sync wait loop */
     if (player->sched == SCHED_EXTERNAL_SYNC)
       player_external_sync(player, 0);
-    
+
     /* Wait until the thread is dead */
     g_thread_join(player->thread);
 
@@ -1023,7 +1031,7 @@ void player_set_position(struct player *player, int position)
     position = player->song->playseqs[player->playseq]->length - 1;
 
   player->position = position;
-  
+
   g_mutex_unlock(player->mutex);
 }
 
@@ -1065,10 +1073,10 @@ void player_set_line(struct player *player, int line)
 void player_set_tick(struct player *player, int tick)
 {
   nullcheck_void(player, player_set_tick);
-  
+
   g_mutex_lock(player->mutex);
   player->tick = tick;
-  g_mutex_unlock(player->mutex);  
+  g_mutex_unlock(player->mutex);
 }
 
 /* Resets the player time */
